﻿using AutoWorkshopContracts.BindingModels;
using AutoWorkshopContracts.BusinessLogicContracts;
using AutoWorkshopContracts.BusinessLogicsContracts;
using AutoWorkshopView.Forms;
using Microsoft.Extensions.Logging;
using System.Windows.Forms;

namespace AutoWorkshopView
{
	public partial class MainForm : Form
    {
        private readonly ILogger _logger;
        private readonly IOrderLogic _orderLogic;
        private readonly IReportLogic _reportLogic;
        private readonly IWorkProcess _workProcess;

        public MainForm(ILogger<MainForm> Logger, IOrderLogic OrderLogic, IReportLogic ReportLogic, IWorkProcess WorkProcess)
        {
            InitializeComponent();

            _logger = Logger;
            _orderLogic = OrderLogic;
            _reportLogic = ReportLogic;
            _workProcess = WorkProcess;
        }

        private void MainForm_Load(object sender, EventArgs e)
        {
            LoadData();
        }

        private void LoadData()
        {
            _logger.LogInformation("Загрузка заказов");

            try
            {
                var List = _orderLogic.ReadList(null);

                if (List != null)
                {
                    DataGridView.DataSource = List;
                    DataGridView.Columns["RepairId"].Visible = false;
					DataGridView.Columns["ClientId"].Visible = false;
					DataGridView.Columns["ClientEmail"].Visible = false;
					DataGridView.Columns["ImplementerId"].Visible = false;
                    DataGridView.Columns["RepairName"].AutoSizeMode = DataGridViewAutoSizeColumnMode.Fill;
                }

                _logger.LogInformation("Загрузка заказов");
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка загрузки заказов");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void ComponentsStripMenuItem_Click(object sender, EventArgs e)
        {
            var Service = Program.ServiceProvider?.GetService(typeof(FormComponents));
            
            if (Service is FormComponents Form)
            {
                Form.ShowDialog();
            }
        }

        private void RepairsStripMenuItem_Click(object sender, EventArgs e)
        {
            var Service = Program.ServiceProvider?.GetService(typeof(FormRepairs));

            if (Service is FormRepairs Form)
            {
                Form.ShowDialog();
            }
        }

        private void CreateOrderButton_Click(object sender, EventArgs e)
        {
            var Service = Program.ServiceProvider?.GetService(typeof(FormCreateOrder));
            
            if (Service is FormCreateOrder Form)
            {
                Form.ShowDialog();
                LoadData();
            }
        }

        private void TakeOrderInWorkButton_Click(object sender, EventArgs e)
        {
            if (DataGridView.SelectedRows.Count == 1)
            {
                int id = Convert.ToInt32(DataGridView.SelectedRows[0].Cells["Id"].Value);
                
                _logger.LogInformation("Заказ №{id}. Меняется статус на 'В работе'", id);
                
                try
                {
                    var OperationResult = _orderLogic.TakeOrderInWork(new OrderBindingModel { Id = id });
                    
                    if (!OperationResult)
                    {
                        throw new Exception("Ошибка при сохранении. Дополнительная информация в логах.");
                    }

                    LoadData();
                }
                catch (Exception ex)
                {
                    _logger.LogError(ex, "Ошибка передачи заказа в работу");
                    MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }

        }

        private void OrderReadyButton_Click(object sender, EventArgs e)
        {
            if (DataGridView.SelectedRows.Count == 1)
            {
                int id = Convert.ToInt32(DataGridView.SelectedRows[0].Cells["Id"].Value);

                _logger.LogInformation("Заказ №{id}. Меняется статус на 'Готов'", id);

                try
                {
                    var OperationResult = _orderLogic.FinishOrder(new OrderBindingModel { Id = id });
                    
                    if (!OperationResult)
                    {
                        throw new Exception("Ошибка при сохранении. Дополнительная информация в логах.");
                    }

                    LoadData();
                }
                catch (Exception ex)
                {
                    _logger.LogError(ex, "Ошибка отметки о готовности заказа");
                    MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        private void OrderDeliveredButton_Click(object sender, EventArgs e)
        {
            if (DataGridView.SelectedRows.Count == 1)
            {
                int id = Convert.ToInt32(DataGridView.SelectedRows[0].Cells["Id"].Value);

                _logger.LogInformation("Заказ №{id}. Меняется статус на 'Выдан'", id);

                try
                {
                    var OperationResult = _orderLogic.DeliveryOrder(new OrderBindingModel { Id = id });
                    
                    if (!OperationResult)
                    {
                        throw new Exception("Ошибка при сохранении. Дополнительная информация в логах.");
                    }
                    
                    _logger.LogInformation("Заказ №{id} выдан", id);
                    LoadData();
                }
                catch (Exception ex)
                {
                    _logger.LogError(ex, "Ошибка отметки о выдачи заказа");
                    MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        private void RefreshButton_Click(object sender, EventArgs e)
        {
            LoadData();
        }

        private void ComponentsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            using var Dialog = new SaveFileDialog { Filter = "docx|*.docx" };
            
            if (Dialog.ShowDialog() == DialogResult.OK)
            {
                _reportLogic.SaveRepairsToWordFile(new ReportBindingModel { FileName = Dialog.FileName });
                MessageBox.Show("Выполнено", "Успех", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
        }

        private void ComponentRepairToolStripMenuItem_Click(object sender, EventArgs e)
        {
            var Service = Program.ServiceProvider?.GetService(typeof(FormReportRepairComponents));
            
            if (Service is FormReportRepairComponents Form)
            {
                Form.ShowDialog();
            }
        }

        private void OrdersToolStripMenuItem_Click(object sender, EventArgs e)
        {
            var Service = Program.ServiceProvider?.GetService(typeof(FormReportOrders));
            
            if (Service is FormReportOrders Form)
            {
                Form.ShowDialog();
            }
        }

		private void ClientsToolStripMenuItem_Click(object sender, EventArgs e)
		{
			var Service = Program.ServiceProvider?.GetService(typeof(FormClients));
			
            if (Service is FormClients Form)
			{
				Form.ShowDialog();
			}
		}

        private void EmployersToolStripMenuItem_Click(object sender, EventArgs e)
        {
            var Service = Program.ServiceProvider?.GetService(typeof(FormImplementers));
            
            if (Service is FormImplementers Form)
            {
                Form.ShowDialog();
            }
        }

        private void StartWorkToolStripMenuItem_Click(object sender, EventArgs e)
        {
            var ImplementerLogic = Program.ServiceProvider?.GetService(typeof(IImplementerLogic));
            _workProcess.DoWork((ImplementerLogic as IImplementerLogic)!, _orderLogic);
            
            MessageBox.Show("Процесс обработки запущен", "Сообщение", MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

		private void MailToolStripMenuItem_Click(object sender, EventArgs e)
		{
			var Service = Program.ServiceProvider?.GetService(typeof(FormMail));
			
            if (Service is FormMail Form)
			{
				Form.ShowDialog();
			}
		}
	}
}
